﻿using System;
using System.Collections.Generic;
using System.Linq;
using SplitAndMerge;

namespace scripting
{
  public class AutoScaleFunction : ParserFunction
  {
    public static int MIN_WIDTH = 640;

    public static double ScaleX { get; private set; }
    public static double ScaleY { get; private set; }

    protected override Variable Evaluate(ParsingScript script)
    {
      bool isList = false;
      List<Variable> args = Utils.GetArgs(script,
          Constants.START_ARG, Constants.END_ARG, out isList);
      Utils.CheckArgs(args.Count, 1, m_name);

      ScaleX = Utils.GetSafeDouble(args, 0);
      ScaleY = Utils.GetSafeDouble(args, 1, ScaleX);

      return Variable.EmptyInstance;
    }

    public static double GetScale(double configOverride, bool isWidth)
    {
      if (configOverride != 0.0) {
        return configOverride;
      }
      return isWidth ? ScaleX : ScaleY;
    }
    public static void TransformSizes(ref int width, ref int height, int screenWidth, string option, double extra = 0.0)
    {
      if (!string.IsNullOrWhiteSpace(option) && option != "auto") {
        return;
      }
      if (extra == 0.0) {
        extra = ScaleX;
        if (extra == 0.0) {
          return;
        }
      }

      int newWidth = TransformSize(width, screenWidth, extra);
      if (width != 0) {
        double ratio = (double)newWidth / (double)width;
        height = (int)(height * ratio);
      } else {
        height = TransformSize(height, screenWidth, extra);
      }
      width = newWidth;

      return;
    }
    public static int TransformSize(int size, int screenWidth, double extra = 1.0)
    {
      if (screenWidth <= MIN_WIDTH) {
        return size;
      }
      return (int)(size * screenWidth * extra / MIN_WIDTH);
    }
  }
  public class InvokeNativeFunction : ParserFunction
  {
    protected override Variable Evaluate(ParsingScript script)
    {
      string methodName = Utils.GetItem(script).AsString();
      Utils.CheckNotEmpty(script, methodName, m_name);
      script.MoveForwardIf(Constants.NEXT_ARG);

      string paramName = Utils.GetToken(script, Constants.NEXT_ARG_ARRAY);
      Utils.CheckNotEmpty(script, paramName, m_name);
      script.MoveForwardIf(Constants.NEXT_ARG);

      Variable paramValueVar = Utils.GetItem(script);
      string paramValue = paramValueVar.AsString();

      var result = Utils.InvokeCall(typeof(Statics),
                                    methodName, paramName, paramValue);
      return result;
    }
  }
  public class GetRandomFunction : ParserFunction
  {
    static Random m_random = new Random();

    protected override Variable Evaluate(ParsingScript script)
    {
      bool isList = false;
      List<Variable> args = Utils.GetArgs(script,
                            Constants.START_ARG, Constants.END_ARG, out isList);
      Utils.CheckArgs(args.Count, 1, m_name);
      int limit = args[0].AsInt();
      Utils.CheckPosInt(args[0]);
      int numberRandoms = Utils.GetSafeInt(args, 1, 1);

      if (numberRandoms <= 1) {
        return new Variable(m_random.Next(0, limit));
      }

      List<int> available = Enumerable.Range(0, limit).ToList();
      List<Variable> result = new List<Variable>();

      for (int i = 0; i < numberRandoms && available.Count > 0; i++) {
        int nextRandom = m_random.Next(0, available.Count);
        result.Add(new Variable(available[nextRandom]));
        available.RemoveAt(nextRandom);
      }

      return new Variable(result);
    }
  }
}
